#!/usr/bin/perl

use strict;
use Socket;
#
# PRS Server and UDP Access Port
#

my $server = "127.0.0.1";
my $port = 1315;
my $call = "NOCALL-10";

#
# The Beacons
#
my @beacons = (

    # My ID
    {
	cycle => 25, offset => 0, type => "DISABLED",
	text => "!0000.00NI00000.00E&PHG2230APRS iGate Server"
    },

    # Objects
    {
	cycle => 10, offset => 1, type => "TNC",
	text => ";145.500-M*111111z0000.01N/00000.00Er145.500MHz Toff R15k Mobile Calling"
    },
    {
	cycle => 30, offset => 1, type => "NET",
	text => ";145.500-M*111111z0000.01N/00000.00Er145.500MHz Toff R15k Mobile Calling"
    },

    {
	cycle => 30, offset => 2, type => "BOTH",
	text => ";NOCALL   *111111z0000.00N/00000.01E-Some Object"
    },

    # Info/POI
    {
	cycle => 10, offset => 3,  type => "TNC",
	text => ";HOSPTL   *111111z0000.01N/00000.01EhEmergency Room"
    },
    {
	cycle => 30, offset => 3,  type => "NET",
	text => ";HOSPTL   *111111z0000.01N/00000.01EhEmergency Room"
    },

    # Repeaters
    {
	cycle => 5, offset => 1,  type => "TNC",
	text => ";NOCALL-1 *111111z0000.02N/00000.00ErPHG4460438.000MHz Toff -760 R40k Some Repeater"
    },
    {
	cycle => 30, offset => 1,  type => "NET",
	text => ";NOCALL-1 *111111z0000.02N/00000.00ErPHG4460438.000MHz Toff -760 R40k Some Repeater"
    },

    {
	cycle => 15, offset => 0,  type => "TNC",
	text => ">Some Beacon Text"
    },

    {
	cycle => 15, offset => 0,  type => "TEMPLATE",
	text => "Text"
    },
);



#
# No user configuration after this
#

my @timer_array;
for my $i (0 .. $#beacons)
{
    push(@timer_array, $beacons[$i]{offset} * 60 + ($i * 3)%55);
}


while (1) 
{
    my $i;
    my $serveraddr;

    for $i (0 .. $#beacons)
    {
	if ($timer_array[$i] == 0)
	{
	    (my $d1, my $d2, my $d3, my $d4, my $rawserver) = gethostbyname($server);
	    $serveraddr = pack("Sna4x8",2,$port,$rawserver);
	    my $prototype = getprotobyname('udp');
	    socket(SOCKET,2,SOCK_DGRAM,$prototype);
	    last;
	}
    }

    for $i (0 .. $#beacons)
    {
	if ($timer_array[$i] == 0)
	{
	    if (($beacons[$i]{type} eq "TNC") || ($beacons[$i]{type} eq "BOTH"))
	    {
		#Send to the TNC... Note: you must end the string with a NULL 
		#Note that only the data after the ":" is sent to the TNC.
		#The message will be sent from node call to APD225
		send SOCKET, "N0CLU>TNC:".$beacons[$i]{text}."\0" , 0 , $serveraddr;
	    }

	    if (($beacons[$i]{type} eq "NET") || ($beacons[$i]{type} eq "BOTH"))
	    {
		#Send  to the Internet...
		#Note that the entire string is sent to the internet including
		#the path "N0CLU>APRS:".
		send SOCKET, $call.">APRS:".$beacons[$i]{text}."\0" , 0 , $serveraddr;
	    }
	    $timer_array[$i] =  $beacons[$i]{cycle} * 60;
	}
    }
    close(SOCKET);

    for $i (0 .. $#beacons)
    {
	$timer_array[$i]--;
    }

    select(undef, undef, undef, 1);
}

